package seguros;

import seguros.*;
import seguros.excepciones.*;
import java.util.*;
import java.sql.*;

import javax.sql.DataSource;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;

/**
 * Clase Datos que implementa algunas funciones de acceso a la Base de datos que
 * no tienen cabida en ningun JavaBean. Por ejemplo, la autentificacin.
 */
public class Datos {
  private DataSource ds;
  
  /**
   * Constructor: Coge una conexin del pool de conexiones
   */
  Datos() throws NamingException {
    // Pool de conexiones a la BD
    // Obtain connection
    InitialContext ctx = new InitialContext();
    ds = (DataSource)ctx.lookup("java:comp/env/jdbc/SegurosDB");
  }
  
  /**
   * Obtiene una conexin del pooling de conexiones a base de datos
   */
  private Connection obtenerConexion() throws SQLException{
    return ds.getConnection();
  }
  
  /**
   * Libera una conexin para que este disponible en el pooling de conexiones a base de datos
   * @param conexion Conexin a base de datos
   */
  private void liberarConexion(Connection conexion) throws SQLException{
    conexion.close();
  }
  
  /**
   * Mtodo privado que realiza una consulta en la BD para comprovar que el login
   * y el password del administrador son validos.
   * @param login Login del administrador
   * @param password Password del administrador
   * @return Devuelve un booleano indicando si es correcto su login/password o no.
   */
  private boolean autorizarAdmin(String login, String password) throws SQLException{
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    sqlLine = "SELECT * FROM administrador WHERE login = ? AND password = PASSWORD(?)"; 
    try {
      ps = conexion.prepareStatement(sqlLine);
      ps.setString(1,login);
      ps.setString(2,password);
      resultado = ps.executeQuery();
      if (resultado.next()) {
        liberarConexion(conexion);
        return true;
      } else {
        liberarConexion(conexion);
        return false;
      }
    } catch (SQLException e) {
      return false;
    }
    
  }
  
  /**
   * Mtodo privado que realiza una consulta en la BD para comprovar que el login
   * y el password del agente son validos.
   * @param login Login del agente
   * @param password Password del agente
   * @return Devuelve un booleano indicando si es correcto su login/password o no.
   */
  private boolean autorizarAgente(String login, String password) throws SQLException{
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    sqlLine = "SELECT * FROM agentes WHERE login = ? AND password = PASSWORD(?)"; 
    try {
      ps = conexion.prepareStatement(sqlLine);
      ps.setString(1,login);
      ps.setString(2,password);
      resultado = ps.executeQuery();
      if (resultado.next()) {
        liberarConexion(conexion);
        return true;
      } else {
        liberarConexion(conexion);
        return false;
      }
    } catch (SQLException e) {
      return false;
    }
    
  }
  
  /**
   * Mtodo privado que realiza una consulta en la BD para comprovar que el login
   * y el password del cliente son validos.
   * @param login Login del cliente
   * @param password Password del cliente
   * @return Devuelve un booleano indicando si es correcto su login/password o no.
   */
  private boolean autorizarCliente(String login, String password) throws SQLException{
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    sqlLine = "SELECT * FROM clientes WHERE login = ? AND password = PASSWORD(?)"; 
    try {
      ps = conexion.prepareStatement(sqlLine);
      ps.setString(1,login);
      ps.setString(2,password);
      resultado = ps.executeQuery();
      if (resultado.next()) {
        liberarConexion(conexion);
        return true;
      } else {
        liberarConexion(conexion);
        return false;
      }
    } catch (SQLException e) {
      return false;
    }
    
  }
  
  /**
   * Realiza una consulta en la BD para comprobar que el login y el password pertenece
   * a un usuario del sistema.
   * @param login Login del usuario
   * @param password Password del usuario
   * @return Devuelve un objeto de la clase TipoUsuario que indica que tipo de usuario es
   * @throws NoValido Si el login y/o password no es correcto se lanza esta excepcin
   */
  public TipoUsuario autorizar(String login, String password) throws NoValido, SQLException{
    
    
    if (! autorizarAdmin(login, password)) {
      if (! autorizarAgente(login, password)) {
        if (! autorizarCliente(login, password)) {
          throw new NoValido();
        } else {
          try {
            return new TipoUsuario("cliente");
          } catch (TipoUsuarioIncorrecto e) {
            return null;
          }
        }
      } else {
        try {
          return new TipoUsuario("agente");
        } catch (TipoUsuarioIncorrecto e) {
          return null;
        }
      }
    } else {
      try {
        return new TipoUsuario("admin");
      } catch (TipoUsuarioIncorrecto e) {
         return null;
      }
    }
    
  }
  
  /**
   * Consulta la informacin del administrador en la BD y devuelve un JavaBean Admin
   * que contiene dicha informacin.
   * @param login Login del administrador
   * @return Devuelve JavaBean Admin
   */
  public Admin getAdmin(String login) throws NoEncontrado,SQLException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    sqlLine = "SELECT * FROM administrador WHERE login = ?"; 
    try {
      ps = conexion.prepareStatement(sqlLine);
      ps.setString(1,login);
      resultado = ps.executeQuery();
      if (resultado.next()) {
        Admin adm = new Admin();
        adm.setId(resultado.getInt("ADM_ID"));
        adm.setLogin(resultado.getString("login"));
        adm.setNombre(resultado.getString("nombre"));
        adm.setTelf(resultado.getInt("telf"));
        adm.setDni(resultado.getInt("dni"));
        adm.setEmail(resultado.getString("mail"));
        liberarConexion(conexion);
        return adm;
      } else {
        throw new NoEncontrado();
      }
    } catch (SQLException e) {
      throw new NoEncontrado();
    }
    
  }
  
  
  /**
   * Consulta la informacin del agente en la BD y devuelve un JavaBean Agente
   * que contiene dicha informacin.
   * @param login Login del agente
   * @return Devuelve JavaBean Agente
   */
  public Agente getAgente(String login) throws NoEncontrado,SQLException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    Agente ag;
    
    sqlLine = "SELECT * FROM agentes WHERE login = ?"; 
    try {
      ps = conexion.prepareStatement(sqlLine);
      ps.setString(1,login);
      resultado = ps.executeQuery();
      if (resultado.next()) {
        ag = new Agente();
        ag.setId(resultado.getInt("AG_ID"));
        ag.setLogin(resultado.getString("login"));
        ag.setNombre(resultado.getString("nombre"));
        ag.setTelf(resultado.getInt("telf"));
        ag.setDni(resultado.getInt("dni"));
        ag.setEmail(resultado.getString("mail"));
      } else {
        throw new NoEncontrado();
      }
      sqlLine = "SELECT CO_ID FROM compania";
      resultado = sentencia.executeQuery(sqlLine);
      if (resultado.next()) {
        ag.setIdCompania(resultado.getString("CO_ID"));
        liberarConexion(conexion);
        return ag;
      } else {
        throw new NoEncontrado();
      }
    } catch (SQLException e) {
      throw new NoEncontrado();
    }
    
  }
  
  /**
   * Consulta la informacin del cliente en la BD y devuelve un JavaBean Cliente
   * que contiene dicha informacin.
   * @param login Login del cliente
   * @return Devuelve JavaBean Cliente
   */
  public Cliente getCliente(String login) throws NoEncontrado,SQLException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    sqlLine = "SELECT * FROM clientes WHERE login = ?"; 
    try {
      ps = conexion.prepareStatement(sqlLine);
      ps.setString(1,login);
      resultado = ps.executeQuery();
      if (resultado.next()) {
        Cliente cl = new Cliente();
        cl.setId(resultado.getInt("CL_ID"));
        cl.setLogin(resultado.getString("login"));
        cl.setNombre(resultado.getString("nombre"));
        cl.setTelf(resultado.getInt("telf"));
        cl.setDni(resultado.getInt("dni"));
        cl.setEmail(resultado.getString("mail"));
        liberarConexion(conexion);
        return cl;
      } else {
        throw new NoEncontrado();
      }
    } catch (SQLException e) {
      throw new NoEncontrado();
    }
    
  }

  // NOTA: Entre clientes, agentes y el admin no puede coincidir nunca su login.
  

  // XML Generator
  
  /**
   * Metodo privado que consulta la URL del DTD para generar XML
   * @return Devuelve un string con la URL del DTD
   */
  public String getDTD() throws NoEncontrado,SQLException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    sqlLine = "SELECT * FROM compania"; 
    try {
      ps = conexion.prepareStatement(sqlLine);
      resultado = ps.executeQuery();
      if (resultado.next()) {
        String campoResultado = resultado.getString("DTD");
        liberarConexion(conexion);
        return campoResultado;
      } else {
        throw new NoEncontrado();
      }
    } catch (SQLException e) {
      throw new NoEncontrado();
    }
    
  }
  
  // Agente
  
  /**
   * Devuelve la URL del servidor homer
   * @return URL
   */
  public String getHomerURL() throws NoEncontrado,SQLException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    sqlLine = "SELECT * FROM compania"; 
    try {
      ps = conexion.prepareStatement(sqlLine);
      resultado = ps.executeQuery();
      if (resultado.next()) {
        String campoResultado = resultado.getString("Homer");
        liberarConexion(conexion);
        return campoResultado;
      } else {
        throw new NoEncontrado();
      }
    } catch (SQLException e) {
      throw new NoEncontrado();
    }
    
  }
  
  /////////// CALL CENTER
  
  /**
   * Da de alta un parte externo
   * @param f Objecto FichaParteExterno a guardar
   * @throws SQLException
   */
  public void nuevoParteExterno(FichaParteExterno f) throws SQLException {
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    sqlLine = "INSERT INTO partes_externos VALUES ('',?,?,?,'INITIAL',?,?,?,'','')";
    ps = conexion.prepareStatement(sqlLine);
    ps.setString(1,f.getIdParteExterno());
    ps.setInt(2,f.getCompaniaAcusadora());
    ps.setInt(3,f.getPolizaCausante());
    ps.setString(4,f.getDescripcion());
    ps.setDate(5,f.getFecha());
    ps.setTime(6,f.getHora());
    ps.executeUpdate();
    
    liberarConexion(conexion);
  }
  
  /**
   * Comproba que el num de una poliza es nuestra
   * @return true si la poliza es valida; false en caso contrario
   */
  public boolean polizaValida(int poliza) throws SQLException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
   
    sqlLine = "SELECT * FROM polizas WHERE PO_ID = ?";
    ps = conexion.prepareStatement(sqlLine);
    ps.setInt(1,poliza);
    resultado = ps.executeQuery();
    if (resultado.next()) {
      liberarConexion(conexion);
      return true;
    } else {
      liberarConexion(conexion);
      return false;
    }
    
  }
  
  /**
   * Devuleve una colecion de ElementoParte especial para la comunicacion XML
   * @return Coleccion de los partes internos donde la poliza indicada es causante
   */
  public Collection partesStatusList(int poliza) throws SQLException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
   
    ElementoParte e;
    Collection c = new Vector();
    String estado = null;
    
    sqlLine = "SELECT * FROM partes_internos WHERE poliza_causante = ?";
    ps = conexion.prepareStatement(sqlLine);
    ps.setInt(1,poliza);
    resultado = ps.executeQuery();
    while (resultado.next()) {
      
      if ((resultado.getString("estado")).equals("INITIAL")){
        estado = "NEED_VALIDATION";
      } else if ((resultado.getString("estado")).equals("RESOLVED")){
        estado = "RESOLVED";
      } else if ((resultado.getString("estado")).equals("VALIDATED")){
        estado = "NEED_RESOLUTION";
      } else if ((resultado.getString("estado")).equals("REJECTED")){
        estado = "RESOLVED";
      }
      e = new ElementoParte(resultado.getString("PI_ID"), estado);
      c.add(e);
    }
    
    liberarConexion(conexion);
    return c;
    
  }
  
  /**
   * Devuelve informacin sobre el agente que se encarga del parte especificado.
   * El parte ha sido generado internamente y el acusado es externo.
   * @return ElementoAgente con la informacin del agente encargado de dicho parte; null si no existe el parte
   */
  public ElementoAgente contactData(int parte) throws SQLException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
   
    ElementoParte e;
    Collection c = new Vector();
    String estado = null;
    
    sqlLine = "SELECT PO_ID FROM partes_internos WHERE PI_ID = ?";
    ps = conexion.prepareStatement(sqlLine);
    ps.setInt(1,parte);
    resultado = ps.executeQuery();
    if (resultado.next()) {
      int poliza = resultado.getInt("PO_ID");
      
      sqlLine = "SELECT CL_ID FROM polizas WHERE PO_ID = ?";
      ps = conexion.prepareStatement(sqlLine);
      ps.setInt(1,poliza);
      resultado = ps.executeQuery();
      
      if (resultado.next()) {
        int cliente = resultado.getInt("CL_ID");
        
        sqlLine = "SELECT AG_ID FROM clientes WHERE CL_ID = ?";
        ps = conexion.prepareStatement(sqlLine);
        ps.setInt(1,cliente);
        resultado = ps.executeQuery();
        
        if (resultado.next()) {
          int agente = resultado.getInt("AG_ID");
          
          sqlLine = "SELECT * FROM agentes WHERE AG_ID = ?";
          ps = conexion.prepareStatement(sqlLine);
          ps.setInt(1,agente);
          resultado = ps.executeQuery();
          
          if (resultado.next()) {
            
            ElementoAgente agentElem = new ElementoAgente(resultado.getString("nombre"), resultado.getInt("telf"), resultado.getString("mail"));
            
            liberarConexion(conexion);
            return agentElem;
          }
        }
      }
    }
    
    liberarConexion(conexion);
    return null;
    
  }
  
  /**
   * Mtodo privado que modifica el estado, indemnizacion y motivoRechazo del parte indicado
   * @param parte Parte a modificar
   * @param estadoActual Estado en el que debe estar el parte actualmente para poder cambiar
   * @param estado Nuevo estado
   * @param indemnizacion Indemnizacion
   * @param motivoRechazo Pq ha sido rechazado.
   * @throws NoEncontrado No encuentra el parte
   * @throws NoValido Desde el estado actual no se puede pasar al nuevo estado
   */
  private void setStatus(int parte, String estadoActual, String estado, int indemnizacion, String motivoRechazo) throws SQLException,NoEncontrado,NoValido {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    // Comprobamos que desde el estado actual se puede pasar el nuevo
    // Si el parte esta en el estado actual, el SELECT nos devolvera 1 resultado, sino nada
    sqlLine = "SELECT * FROM partes_internos WHERE PI_ID = ? AND estado = ?";
    ps = conexion.prepareStatement(sqlLine);
    ps.setInt(1,parte);
    ps.setString(2,estadoActual);
    resultado = ps.executeQuery();
      
    // Si no esta en ese estado (estadoActual), no puede migrar al nuevo
    if (! resultado.next()) {
      throw new NoValido();
    }
        
    sqlLine = "UPDATE partes_internos SET estado = ?, indemnizacion = ?, descripcion_rechazo = ? WHERE PI_ID = ?";
    ps = conexion.prepareStatement(sqlLine);
    ps.setString(1,estado);
    ps.setInt(2,indemnizacion);
    ps.setString(3,motivoRechazo);
    ps.setInt(4,parte);
    // Si no hay ninguna modificacion es que no se ha encontrado
    if (ps.executeUpdate() == 0) {
      throw new NoEncontrado();
    }
    
    liberarConexion(conexion);
  }
  
  /**
   * Cambia el estado a Resolved del parte especificado con la indemnizacin indicada.
   * @param e ElementoResolved con la informacin
   * @throws NoEncontrado si no encuentra el parte
   * @throws NoValido Desde el estado actual no se puede pasar al nuevo estado
   */
  public void setStatusResolved(ElementoResolved e) throws SQLException,NoEncontrado,NoValido {
    setStatus(e.getCode(), "VALIDATED", "RESOLVED", e.getCantidad(), "");
  }
  
  /**
   * Cambia el estado a Validated del parte especificado 
   * @param code Cdigo del parte
   * @throws NoEncontrado si no encuentra el parte
   * @throws NoValido Desde el estado actual no se puede pasar al nuevo estado
   */
  public void setStatusValidated(int code) throws SQLException,NoEncontrado,NoValido {
    setStatus(code, "INITIAL", "VALIDATED", 0, "");
  }
  
  /**
   * Cambia el estado a Rejected del parte especificado con el motivo indicada.
   * @param e ElementoRejected con la informacin
   * @throws NoEncontrado si no encuentra el parte
   * @throws NoValido Desde el estado actual no se puede pasar al nuevo estado
   */
  public void setStatusRejected(ElementoRejected e) throws SQLException,NoEncontrado,NoValido {
    setStatus(e.getCode(), "VALIDATED", "REJECTED", 0, e.getMotivo());
  }
}